import * as yargs from 'yargs';
import { cli } from '../cli';
import { SSHService } from '../session/ssh';
import fsExtra = require('fs-extra');
import path = require('path');
import { timeout } from '../utils/promises';
import chalk from 'chalk';

export const command = 'deploy';
export const desc = 'Deploy an assistant';

exports.builder = {
    snipsfile_path: {
        demandOption: false,
        describe: 'Location of the Snipsfile',
        type: 'string'
    },
    verbose: {
        demandOption: false,
        describe: 'Include remote logs',
        type: 'boolean'
    }
}

export const handler = async (argv: yargs.Argv) => {
    let snipsfilePath = JSON.parse(JSON.stringify(argv)).snipsfile_path;
    const verbose = JSON.parse(JSON.stringify(argv)).verbose;

    if (snipsfilePath === undefined) {
        snipsfilePath = path.join(process.cwd(), 'Snipsfile');
    }
    
    const ssh = new SSHService();
    await ssh.connect().catch(e => cli.stream.error(e));

    cli.stream.loading(`Deploying assistant to ${ssh.credentials.hostname}...`);
    await timeout(1000);

    const snipsfile = await fsExtra.readFile(snipsfilePath, 'utf8')
        .catch((e) => {
            cli.stream.done(false);
            cli.stream.errorPlain(`Error: Could not locate Snipsfile`);
            process.exit();
        });

    if (snipsfile === undefined) {
        cli.stream.done(false);
        cli.stream.errorPlain(`Error: Could not locate Snipsfile`);
        return process.exit();
    }

    cli.stream.done(false);
    cli.stream.loading(`Copying Snipsfile to device...`);
    await timeout(1000);

    await ssh.copySnipsfile(snipsfile)
        .catch((e) => {
            cli.stream.done(false);
            cli.stream.errorPlain(`Error deploying assistant: ${e.message}`);
            ssh.disconnect();
            process.exit();
        });

    cli.stream.done();
    cli.stream.loading('Installing dependencies. This may take a few minutes...');
    
    await ssh.runSnipsmanagerInstall((output) => {
        if (verbose) {
            cli.stream.done();
            cli.stream.println(output);
        }
    });

    cli.stream.loading('Adding assistant as a systemd service...');
    await timeout(1000);

    cli.stream.done();

    cli.stream.loading('Starting assistant...');
    
    await ssh.runSnipsManager((output) => {
        cli.stream.done(true);
        if (verbose) {
            cli.stream.println(output);
        }
    });

    const assistant = await ssh.getInstalledAssistant();

    cli.stream.done();
    cli.stream.println();

    if (assistant !== undefined) {
        cli.stream.success(`Snips assistant is now running. Say ${chalk.blue(assistant.hotword)} to start!`);
    } else {
        cli.stream.success(`Snips assistant is now running`);
    }
    
    process.exit();
};
